@@ -29,14 +29,13 @@ import dateutil.parser
 import jsonschema
 from selenium import webdriver
 from selenium.common.exceptions import NoSuchElementException
-from selenium.common.exceptions import NoAlertPresentException
-from selenium.common.exceptions import UnexpectedAlertPresentException
 from selenium.webdriver.chrome.options import Options
 import selenium.webdriver.support.select
 
 import cloudfeaster_extension
 from . import jsonschemas
 from . import privacy
+from . import util
 
 _logger = logging.getLogger(__name__)
 
@@ -390,7 +389,7 @@ class CLICrawlArgs(list):
 
 
 class CrawlResponse(dict):
-    """Instances of this class are returned by ```Spider.crawl```."""
+    """Instances of this class are returned by ```Spider.crawl()```."""
 
     SC_OK = 0
     SC_CRAWL_RAISED_EXCEPTION = 400 + 1
@@ -413,12 +412,6 @@ class CrawlResponse(dict):
         }
         dict.__init__(self, *args, **kwargs)
 
-    def add_debug(self, key, value):
-        if '_debug' not in self:
-            self['_debug'] = {}
-
-        self['_debug'][key] = value
-
     @property
     def status_code(self):
         return self.get('_metadata', {}).get('status', {}).get('code', type(self).SC_UNKNOWN)
@@ -571,17 +564,18 @@ class SpiderCrawler(object):
                 if not isinstance(crawl_response, CrawlResponse):
                     crawl_response = CrawlResponseInvalidCrawlReturnType()
 
-                crawl_response = self._take_screenshot(browser, crawl_response)
+                self.screenshot_file = self._take_screenshot(browser)
         except Exception as ex:
             crawl_response = CrawlResponseCrawlRaisedException(ex)
         dt_end = _utc_now()
 
         crawl_response['_metadata'].update({
             'spider': {
-                'name': '%s.%s' % (type(spider).__module__, type(spider).__name__),
+                'name': os.path.basename(sys.modules[type(spider).__module__].__file__),
                 'version': spider_class.version(),
             },
             'crawlArgs': [
+                privacy.hash_crawl_arg(arg) for arg in args
             ],
             'crawlTime': {
                 'started': dt_start.isoformat(),
@@ -589,14 +583,20 @@ class SpiderCrawler(object):
             },
         })
 
-        if self.logging_file:
-            crawl_response.add_debug('crawlLog', self.logging_file)
+        self._add_debug_file_to_crawl_response(
+            'screenshot',
+            self._file_to_data_uri_scheme(self.screenshot_file),
+            crawl_response)
 
-        if self.chromedriver_log_file:
-            crawl_response.add_debug('chromeDriverLog', self.chromedriver_log_file)
+        self._add_debug_file_to_crawl_response(
+            'crawlLog',
+            self._file_to_data_uri_scheme(self.logging_file),
+            crawl_response)
 
-        for arg in args:
-            crawl_response['_metadata']['crawlArgs'].append(privacy.hash_crawl_arg(arg))
+        self._add_debug_file_to_crawl_response(
+            'chromeDriverLog',
+            self._file_to_data_uri_scheme(self.chromedriver_log_file),
+            crawl_response)
 
         #
         # verify ```crawl_response```
@@ -608,6 +608,20 @@ class SpiderCrawler(object):
 
         return crawl_response
 
+    def _file_to_data_uri_scheme(self, filename):
+        is_inline_debug = True if os.environ.get('CLF_INLINE_DEBUG', None) else False
+        if not is_inline_debug:
+            return filename
+
+        return util.file_to_data_uri_scheme(filename)
+
+    def _add_debug_file_to_crawl_response(self, key, value, crawl_response):
+        if key and value:
+            if '_debug' not in crawl_response:
+                crawl_response['_debug'] = {}
+
+            crawl_response['_debug'][key] = value
+
     def _get_spider_class(self):
         #
         # deal with scenario where self.full_spider_class_name
@@ -663,17 +677,16 @@ class SpiderCrawler(object):
             return RemoteBrowser(remote_chromedriver, url, paranoia_level)
         return Browser(url, paranoia_level, chromedriver_log_file)
 
-    def _take_screenshot(self, browser, crawl_response):
+    def _take_screenshot(self, browser):
         """This is a private method which takes a screenshot of the browser's
         current window and then adds the name of the temp file containing the
         screenshot to the crawl response.
         """
         (_, no_extension_screenshot) = tempfile.mkstemp()
-        self.screenshot_file = no_extension_screenshot + '.png'
-        os.rename(no_extension_screenshot, self.screenshot_file)
-        browser.save_screenshot(self.screenshot_file)
-        crawl_response.add_debug('screenshot', self.screenshot_file)
-        return crawl_response
+        screenshot_file = no_extension_screenshot + '.png'
+        os.rename(no_extension_screenshot, screenshot_file)
+        browser.save_screenshot(screenshot_file)
+        return screenshot_file
 
     def _configure_logging(self, crawl_args):
         clf_debug_value = os.environ.get('CLF_DEBUG', '')
@@ -768,9 +781,16 @@ class Browser(webdriver.Chrome):
             _logger.info('using chrome binary >>>%s<<<', chrome_options.binary_location)
 
         # re '--no-sandbox' - see https://github.com/theintern/intern/issues/878
+        chrome_options_str_format = (
+            '--headless|'
+            '--window-size=1280x1024|'
+            '--no-sandbox|'
+            '--disable-dev-shm-usage|'
+            '--user-agent={user_agent}'
+        )
         chrome_options_str = os.environ.get(
             'CLF_CHROME_OPTIONS',
-            '--headless|--window-size=1280x1024|--no-sandbox|--user-agent=%s' % cloudfeaster_extension.user_agent())
+            chrome_options_str_format.format(user_agent=cloudfeaster_extension.user_agent()))
         for chrome_option in chrome_options_str.split('|'):
             chrome_options.add_argument(chrome_option)
             _logger.info('using chrome option >>>%s<<<', chrome_option)
@@ -833,84 +853,6 @@ class Browser(webdriver.Chrome):
         """
         return WebElement(self._paranoia_level, self, element_id)
 
-    def wait_for_login_to_complete(self,
-                                   ok_xpath_locator,
-                                   bad_credentials_xpath_locator=None,
-                                   account_locked_out_xpath_locator=None,
-                                   alert_displayed_indicates_bad_credentials=False,
-                                   number_seconds_until_timeout=30):
-
-        for i in range(0, number_seconds_until_timeout):
-            if self._find_element_by_xpath(ok_xpath_locator):
-                return None
-
-            if bad_credentials_xpath_locator:
-                if self._find_element_by_xpath(bad_credentials_xpath_locator):
-                    return CrawlResponseBadCredentials()
-
-            if alert_displayed_indicates_bad_credentials:
-                if self._is_alert_dialog_displayed():
-                    return CrawlResponseBadCredentials()
-
-            if account_locked_out_xpath_locator:
-                if self._find_element_by_xpath(account_locked_out_xpath_locator):
-                    return CrawlResponseAccountLockedOut()
-
-            time.sleep(_one_second)
-
-        return CrawlResponseCouldNotConfirmLoginStatus()
-
-    def wait_for_signin_to_complete(self,
-                                    ok_xpath_locator,
-                                    bad_credentials_xpath_locator=None,
-                                    account_locked_out_xpath_locator=None,
-                                    alert_displayed_indicates_bad_credentials=None,
-                                    number_seconds_until_timeout=30):
-        """This method is just another name for
-        :py:meth:`wait_for_login_to_complete` so that spider authors
-        can use "login" or "signin" terminology to match
-        the semantics of the web site being crawled.
-        """
-        rv = self.wait_for_login_to_complete(
-            ok_xpath_locator,
-            bad_credentials_xpath_locator,
-            account_locked_out_xpath_locator,
-            alert_displayed_indicates_bad_credentials,
-            number_seconds_until_timeout)
-        return rv
-
-    def _find_element_by_xpath(self, xpath_locator):
-        try:
-            self.find_element_by_xpath(xpath_locator)
-        except NoSuchElementException:
-            return False
-        except UnexpectedAlertPresentException:
-            return False
-
-        return True
-
-    def _is_alert_dialog_displayed(self):
-        """Only to be used by CLF infrastructure.
-
-        Couldn't find a good way to test if a JavaScript alert
-        dialog was displayed and hence the creation of this
-        method. A word of warning. Don't like the implementation
-        of this method because it calls switch_to_alert() which
-        will (surprise, surprise) switch focus to the alert dialog
-        if the dialog is displayed which is almost certainly not
-        the desired behaviour. Because of this side effect it's
-        probably only wise to use this method when precense of
-        alert dialog indicates an error and your spider will be
-        terminated if the alert is displayed.
-        """
-        try:
-            alert = self.switch_to_alert()
-            alert.text
-        except NoAlertPresentException:
-            return False
-
-        return True
-
 
 class WebElement(selenium.webdriver.remote.webelement.WebElement):
     """This class extends ```selenium.webdriver.remote.webelement.WebElement```
@@ -925,6 +867,14 @@ class WebElement(selenium.webdriver.remote.webelement.WebElement):
 
         self._paranoia_level = paranoia_level
 
+    def __eq__(self, other):
+        """This is here only to resolve https://lgtm.com/rules/9990086/."""
+        if not isinstance(other, WebElement):
+            return False
+        if not selenium.webdriver.remote.webelement.WebElement.__eq__(self, other):
+            return False
+        return self._paranoia_level == other._paranoia_level
+
     def get_text(self):
         """This method exists so spider code can access element data
         using a set of methods instead of a text property and some
